--	Player class colors.
function whoaUnitClass(healthbar, unit)
	if UnitIsPlayer(unit) and UnitIsConnected(unit) and UnitClass(unit) and (cfg.classColor) then
		_, class = UnitClass(unit);
		local c = RAID_CLASS_COLORS[class];
		healthbar:SetStatusBarColor(c.r, c.g, c.b);
	elseif UnitIsPlayer(unit) and (not UnitIsConnected(unit)) then
		healthbar:SetStatusBarColor(0.5,0.5,0.5);
	else
		healthbar:SetStatusBarColor(0,0.9,0);
	end
end
hooksecurefunc("UnitFrameHealthBar_Update", whoaUnitClass)
hooksecurefunc("HealthBar_OnValueChanged", function(self)
	whoaUnitClass(self, self.unit)
end)

	--	Blizzard´s target unit reactions HP color
function npcReactionBrightColors()
	if cfg.BlizzardReactionColor == true then
		FACTION_BAR_COLORS = {
			[1] = {r =  0.9, g = 0.0, b = 0.0},
			[2] = {r =  0.9, g = 0.0, b = 0.0},
			[3] = {r =  0.9, g = 0.0, b = 0.0},
			[4] = {r =  1, g =  0.8, b = 0.0},
			[5] = {r = 0.0, g = 0.9, b = 0.0},
			[6] = {r = 0.0, g = 0.9, b = 0.0},
			[7] = {r = 0.0, g = 0.9, b = 0.0},
			[8] = {r = 0.0, g = 0.9, b = 0.0},
		};
	end
end
hooksecurefunc("TargetFrame_CheckFaction", npcReactionBrightColors)

--	Unit faction colors.
function whoaUnitReaction(healthbar, unit)
	if UnitExists(unit) and (not UnitIsPlayer(unit)) and (cfg.reactionColor) then
		if (UnitIsTapDenied(unit)) and not UnitPlayerControlled(unit) then
			healthbar:SetStatusBarColor(0.5, 0.5, 0.5)
		elseif (not UnitIsTapDenied(unit)) then
			local reaction = FACTION_BAR_COLORS[UnitReaction(unit,"player")];
			if reaction then
				healthbar:SetStatusBarColor(reaction.r, reaction.g, reaction.b);
			else
				healthbar:SetStatusBarColor(0,0.6,0.1)
			end
		end
	end
end
hooksecurefunc("UnitFrameHealthBar_Update", whoaUnitReaction)
hooksecurefunc("HealthBar_OnValueChanged", function(self)
	whoaUnitReaction(self, self.unit)
end)

function whoaNoClick()
	-- unclickeable frames
	if (cfg.noClickFrame == true) then
		PlayerFrame:SetMouseClickEnabled(false);
		PetFrame:SetMouseClickEnabled(false);
		TargetFrame:SetMouseClickEnabled(false);
	elseif (cfg.noClickFrame == false) then
		PlayerFrame:SetMouseClickEnabled(true);
		PetFrame:SetMouseClickEnabled(true);
		TargetFrame:SetMouseClickEnabled(true);
	end
end

--	Aura positioning constants.
local LARGE_AURA_SIZE = cfg.largeAuraSize;		--	Default 21.
local SMALL_AURA_SIZE = cfg.smallAuraSize;		--	Default 17.
local AURA_OFFSET_Y = 3;
local AURA_ROW_WIDTH = 122;
local NUM_TOT_AURA_ROWS = 2;

--	Set aura size.
function whoaAuraResize(self, auraName, numAuras, numOppositeAuras, largeAuraList, updateFunc, maxRowWidth, offsetX, mirrorAurasVertically)
	local size;
	local offsetY = AURA_OFFSET_Y;
	local rowWidth = 0;
	local firstBuffOnRow = 1;
	for i=1, numAuras do
		if ( largeAuraList[i] ) then
			size = cfg.largeAuraSize;
			offsetY = AURA_OFFSET_Y + AURA_OFFSET_Y;
		else
			size = cfg.smallAuraSize;
		end
		if ( i == 1 ) then
			rowWidth = size;
			self.auraRows = self.auraRows + 1;
		else
			rowWidth = rowWidth + size + offsetX;
		end
		if ( rowWidth > maxRowWidth ) then
			updateFunc(self, auraName, i, numOppositeAuras, firstBuffOnRow, size, offsetX, offsetY, mirrorAurasVertically);

			rowWidth = size;
			self.auraRows = self.auraRows + 1;
			firstBuffOnRow = i;
			offsetY = AURA_OFFSET_Y;

			if ( self.auraRows > NUM_TOT_AURA_ROWS ) then
				maxRowWidth = AURA_ROW_WIDTH;
			end
		else
			updateFunc(self, auraName, i, numOppositeAuras, i - 1, size, offsetX, offsetY, mirrorAurasVertically);
		end
	end
end
hooksecurefunc("TargetFrame_UpdateAuraPositions", whoaAuraResize)


function customStatusTex(statusFrame, textString, value, valueMin, valueMax)
	local xpValue = UnitXP("player");
	local xpMaxValue = UnitXPMax("player");
	
	if( statusFrame.LeftText and statusFrame.RightText ) then
		statusFrame.LeftText:SetText("");
		statusFrame.RightText:SetText("");
		statusFrame.LeftText:Hide();
		statusFrame.RightText:Hide();
	end
	
	local k,m=1e3
	local m=k*k
	
	valueDisplay	=	((value>1e3 and value<1e5 and format("%1.3f",value/k)) or 
						(value>=1e5 and value<1e6 and format("%1.0f K",value/k)) or 
						(value>=1e6 and value<1e9 and format("%1.1f M",value/m)) or 
						(value>=1e9 and format("%1.1f M",value/m)) or value )
						
	valueMaxDisplay	=	((valueMax>1e3 and valueMax<1e5 and format("%1.3f",valueMax/k)) or
						(valueMax>=1e5 and valueMax<1e6 and format("%1.0f K",valueMax/k)) or 
						(valueMax>=1e6 and valueMax<1e9 and format("%1.1f M",valueMax/m)) or 
						(valueMax>=1e9 and format("%1.1f M",valueMax/m)) or valueMax )					
	
	-- xpValueDisplay	=	( xpValue >= 1e3 and format("%1.3f",xpValue/k) or xpValue)
	
	-- xpMaxValueDisplay	=	( xpMaxValue >= 1e3 and format("%1.3f",xpMaxValue/k) or xpMaxValue)
	
	if ( ( tonumber(valueMax) ~= valueMax or valueMax > 0 ) and not ( statusFrame.pauseUpdates ) ) then
		statusFrame:Show();
		
		if ( (statusFrame.cvar and GetCVar(statusFrame.cvar) == "1" and statusFrame.textLockable) or statusFrame.forceShow ) then
			textString:Show();
		elseif ( statusFrame.lockShow > 0 and (not statusFrame.forceHideText) ) then
			textString:Show();
		else
			textString:SetText("");
			textString:Hide();
			return;
		end

		local textDisplay = GetCVar("statusTextDisplay");
		if ( value and valueMax > 0 and ( (textDisplay ~= "NUMERIC" and textDisplay ~= "NONE") or statusFrame.showPercentage ) and not statusFrame.showNumeric) then
			if ( value == 0 and statusFrame.zeroText ) then
				textString:SetText(statusFrame.zeroText);
				statusFrame.isZero = 1;
				textString:Show();
			elseif ( textDisplay == "BOTH" and not statusFrame.showPercentage) then
				if( statusFrame.LeftText and statusFrame.RightText ) then
					if(not statusFrame.powerToken or statusFrame.powerToken == "MANA") then
						statusFrame.LeftText:SetText(math.ceil((value / valueMax) * 100) .. "%");
						if value == 0 then statusFrame.LeftText:SetText(""); end
						statusFrame.LeftText:Show();
					end
					statusFrame.RightText:SetText(valueDisplay);
					if value == 0 then statusFrame.RightText:SetText(""); end
					statusFrame.RightText:Show();
					textString:Hide();
				else
					valueDisplay = "(" .. math.ceil((value / valueMax) * 100) .. "%) " .. valueDisplay .. " / " .. valueMaxDisplay;
					if value == 0 then textString:SetText(""); end
				end
				textString:SetText(valueDisplay);
			else
				valueDisplay = math.ceil((value / valueMax) * 100) .. "%";
				if ( statusFrame.prefix and (statusFrame.alwaysPrefix or not (statusFrame.cvar and GetCVar(statusFrame.cvar) == "1" and statusFrame.textLockable) ) ) then
					textString:SetText(statusFrame.prefix .. " " .. valueDisplay);
				else
					textString:SetText(valueDisplay);
				end
				if value == 0 then textString:SetText(""); end
			end
		elseif ( value == 0 and statusFrame.zeroText ) then
			textString:SetText(statusFrame.zeroText);
			statusFrame.isZero = 1;
			textString:Show();
			return;
		else
			statusFrame.isZero = nil;
			if ( statusFrame.prefix and (statusFrame.alwaysPrefix or not (statusFrame.cvar and GetCVar(statusFrame.cvar) == "1" and statusFrame.textLockable) ) ) then
				textString:SetText(statusFrame.prefix.." "..valueDisplay.." / "..valueMaxDisplay)
			else
				textString:SetText(valueDisplay.." / "..valueMaxDisplay);
			end
			if value == 0 then textString:SetText("") end
		end
	else
		textString:Hide();
		textString:SetText("");
		if ( not statusFrame.alwaysShow ) then
			statusFrame:Hide();
		else
			statusFrame:SetValue(0);
		end
	end
end
hooksecurefunc("TextStatusBar_UpdateTextStringWithValues",customStatusTex)

--	Custom status text format.
-- hooksecurefunc("TextStatusBar_UpdateTextStringWithValues",function(self,_,value,_,maxValue)
	-- if self.RightText and value and maxValue>0 and not self.showPercentage and GetCVar("statusTextDisplay")=="BOTH" then
		-- local k,m=1e3 
		-- m=k*k
		-- self.RightText:SetText((value>1e3 and value<1e5 and format("%1.3f",value/k)) or (value>=1e5 and value<1e6 and format("%1.0f K",value/k)) or (value>=1e6 and value<1e9 and format("%1.1f M",value/m)) or (value>=1e9 and format("%1.1f M",value/m)) or value )
		-- if value == 0 then
			-- self.RightText:SetText(" ");
		-- end
	-- end
	-- print(value);
-- end)